# IICO/VMO -- see IT, feel IT, love IT
# Copyright 2020 The IICO Corporate. All Rights Reserved.
#
# ============================================================================
""" 2D Bounding Box Utilities """

from typing import Union
import numpy as np


def yolo2coco(
    boxes: np.ndarray, imsize: tuple, trim: bool = True
) -> np.ndarray:
    """ convert from yolo format to coco format
    Args:
        box: shape of `[num_box, 4]`. The 4 box ratios of format cx, cy, w, h
    Returns:
        shape of `[num_box, 4]`. The 4 box coordinates of format x1, y1, w, h
        If trim == True,  Any negative value will be cliped to 0.
                        Width will be limited to inside boundary.
                        Height will be limited to inside boundary.
    """
    boxes_new = boxes.copy()
    boxes_new = boxes_new.astype(float)
    boxes_new[:, 0] -= 0.5 * boxes[:, 2]
    boxes_new[:, 1] -= 0.5 * boxes[:, 3]
    boxes_new *= np.array([imsize[1], imsize[0], imsize[1], imsize[0]])
    if trim:
        boxes_new = boxes_new.clip(min=0.0)
        for i in range(boxes_new.shape[0]):
            boxes_new[i, 2] = min(
                boxes_new[i, 2], imsize[1] - boxes_new[i, 0] - 1
            )
            boxes_new[i, 3] = min(
                boxes_new[i, 3], imsize[0] - boxes_new[i, 1] - 1
            )
    return boxes_new


def coco2albu(
    boxes: np.ndarray, imsize: tuple, trim: bool = True
) -> np.ndarray:
    """ convert from coco format to albumentation box format
    Args:
        box: shape of `[num_box, 4]`. The 4 box coordinates of format
        [x1, y1, w, h]

    Returns:
        shape of `[num_box, 4]`. The box of format [x1, y1, x2, y2]
        in ratios.
        If trim==True,  Any negative value will be cliped to 0.
                        Any out of bound value will be clipped to boundary-1
    """
    if not min(imsize) > 1.0:
        raise ValueError("[im2d.bbox.coco2albu] Image size error!")
    boxes_new = boxes.copy()
    boxes_new = boxes_new.astype(float)
    boxes_new[:, 2] += boxes_new[:, 0]
    boxes_new[:, 3] += boxes_new[:, 1]
    if trim:
        for i in range(boxes_new.shape[0]):
            boxes_new[i, 0] = max(boxes_new[i, 0], 0.0)
            boxes_new[i, 1] = max(boxes_new[i, 1], 0.0)
            boxes_new[i, 2] = min(boxes_new[i, 2], imsize[1] - 1)
            boxes_new[i, 3] = min(boxes_new[i, 3], imsize[0] - 1)
    boxes_new *= np.array(
        [1 / imsize[1], 1 / imsize[0], 1 / imsize[1], 1 / imsize[0]]
    )
    return boxes_new


def yolo2albu(
    boxes: np.ndarray, imsize: tuple, trim: bool = True
) -> np.ndarray:
    """ convert from yolo format to coco format
    Args:
        box: shape of `[num_box, 4]`. The 4 box ratios of format cx, cy, w, h
    Returns:
        shape of `[num_box, 4]`. The box of format [x1, y1, x2, y2]
        in ratios.
        If trim==True,  Any negative value will be cliped to 0.
                        Any out of bound value will be clipped to boundary-1
    """
    boxes_new = boxes.copy()
    boxes_new = boxes_new.astype(float)
    boxes_new[:, 0] -= 0.5 * boxes[:, 2]
    boxes_new[:, 1] -= 0.5 * boxes[:, 3]
    boxes_new[:, 2] = boxes[:, 0] + 0.5 * boxes[:, 2]
    boxes_new[:, 3] = boxes[:, 1] + 0.5 * boxes[:, 3]

    if trim:
        if not min(imsize) > 1.0:
            raise ValueError("[im2d.bbox.yolo2albu] Image size error!")
        boxes_new *= np.array([imsize[1], imsize[0], imsize[1], imsize[0]])
        for i in range(boxes_new.shape[0]):
            boxes_new[i, 0] = max(boxes_new[i, 0], 0.0)
            boxes_new[i, 1] = max(boxes_new[i, 1], 0.0)
            boxes_new[i, 2] = min(boxes_new[i, 2], imsize[1] - 1)
            boxes_new[i, 3] = min(boxes_new[i, 3], imsize[0] - 1)
        boxes_new *= np.array(
            [1 / imsize[1], 1 / imsize[0], 1 / imsize[1], 1 / imsize[0]]
        )
    return boxes_new


class UniviewBBox(object):
    """ Uniview bounding box uses Albumentation bounding box format.
        This class convert bounding boxes objects to albu format.
        The folliwng formats are considered:
        - pascal_voc: [x_min, y_min, x_max, y_max], e.g. [98, 345, 420, 462]
        - albumentations: [x_min, y_min, x_max, y_max],
                          e.g. [0.153125, 0.71875, 0.65625, 0.9625]
        - coco = 2: [x_min, y_min, width, height], e.g. [98, 345, 322, 117]
        - yolo = 3: [x_center, y_center, width, height],
                    e.g. [0.4046875, 0.8614583, 0.503125, 0.24375]
    """

    conversions = {"coco": coco2albu, "yolo": yolo2albu}

    def __init__(
        self,
        need_trim: bool = True,
        bbox: Union[list, np.ndarray] = None,
        in_format: str = None,
        imsize: tuple = None,
    ):
        """
        Args:
            bbox: bounding box instance in list of 4 float values:
                [b0, b1, b2, b3]
            imsize: a tuple in (height, width, depth) or (height, width)
            in_format: input bbox format in one of
                        [pascal_voc, albumentations, coco, yolo]
        """
        self.bbox = None
        self.trim = need_trim
        if bbox:
            self._to_albu(bbox, in_format, imsize)

    def _to_albu(
        self, bbox: Union[list, np.ndarray], in_format: str, imsize: tuple
    ):
        """
        Args:
            bbox: bounding box instance in list of 4 float values:
                [b0, b1, b2, b3]
            imsize: a tuple in (height, width, depth) or (height, width)
            in_format: input bbox format in one of
                        [pascal_voc, albumentations, coco, yolo]
        """
        if in_format is None:
            raise ValueError(
                "[im2d.bbox.UniviewBBox] Need input format has to process!"
            )
        elif in_format not in ["yolo", "coco"]:
            raise ValueError(
                "[im2d.bbox.UniviewBBox] Method not implemented yet!"
            )
        if bbox is None:
            raise ValueError(
                "[im2d.bbox.UniviewBBox] No valid data to process!"
            )
        if not min(imsize) > 1.0:
            raise ValueError("[im2d.bbox.UniviewBBox] Image size error!")
        if isinstance(bbox, list) and not isinstance(bbox[0], (list, tuple)):
            bbox = [bbox]
        if not isinstance(bbox, np.ndarray):
            bbox = np.array(bbox)
        self.bbox = self.conversions[in_format](bbox, imsize, trim=self.trim)

    def get_albu_bbox(
        self, bbox: Union[list, np.ndarray], in_format: str, imsize: tuple
    ):
        self._to_albu(bbox, in_format, imsize)
        return self.get_bbox()

    def get_bbox(self):
        """
        Return:
            numpy array in albumentation bbox format
        """
        return self.bbox



from enum import Enum


class JointFormat(Enum):
    coco = 0
    openpose = 1
    openpose20 = 2
    openpose22 = 3
    vmo = 4
    vmo16 = 5
    vmo18 = 6
    torso = 7


class TorsoPart(Enum):
    Head = 0
    RShoulder = 1
    LShoulder = 2
    Hip = 3


class VmoPart(Enum):
    Head = 0
    Neck = 1
    RShoulder = 2
    RElbow = 3
    RWrist = 4
    LShoulder = 5
    LElbow = 6
    LWrist = 7
    RHip = 8
    RKnee = 9
    RAnkle = 10
    LHip = 11
    LKnee = 12
    LAnkle = 13


class CocoPart(Enum):
    Nose = 0
    LEye = 1
    REye = 2
    LEar = 3
    REar = 4
    LShoulder = 5
    RShoulder = 6
    LElbow = 7
    RElbow = 8
    LWrist = 9
    RWrist = 10
    LHip = 11
    RHip = 12
    LKnee = 13
    RKnee = 14
    LAnkle = 15
    RAnkle = 16


class Coco13Part(Enum):
    Head = 0
    LShoulder = 1
    RShoulder = 2
    LElbow = 3
    RElbow = 4
    LWrist = 5
    RWrist = 6
    LHip = 7
    RHip = 8
    LKnee = 9
    RKnee = 10
    LAnkle = 11
    RAnkle = 12


class OpenposePart(Enum):
    Nose = 0
    Neck = 1
    RShoulder = 2
    RElbow = 3
    RWrist = 4
    LShoulder = 5
    LElbow = 6
    LWrist = 7
    RHip = 8
    RKnee = 9
    RAnkle = 10
    LHip = 11
    LKnee = 12
    LAnkle = 13
    REye = 14
    LEye = 15
    REar = 16
    LEar = 17
    Background = 18


coco13_flip_list = [
    Coco13Part.Head,
    Coco13Part.RShoulder,
    Coco13Part.LShoulder,
    Coco13Part.RElbow,
    Coco13Part.LElbow,
    Coco13Part.RWrist,
    Coco13Part.LWrist,
    Coco13Part.RHip,
    Coco13Part.LHip,
    Coco13Part.RKnee,
    Coco13Part.LKnee,
    Coco13Part.RAnkle,
    Coco13Part.LAnkle,
]


nr_torso_keypoints = 4  # 4 joints
torso_skeleton = [(0, 1), (0, 2), (0, 3), (1, 2), (2, 3), (1, 3)]
torso_flip_list = [
    TorsoPart.Head,
    TorsoPart.LShoulder,
    TorsoPart.RShoulder,
    TorsoPart.Hip,
]
nr_torso_links = 5
torsopose_vecs = list(zip([0, 0, 1, 1, 2], [1, 2, 2, 3, 3]))

nr_vmo_keypoints = 14  # 14 joints
vmo_skeleton = [
    (0, 1),
    (1, 2),
    (2, 3),
    (3, 4),
    (1, 5),
    (5, 6),
    (6, 7),
    (1, 8),
    (8, 9),
    (9, 10),
    (1, 11),
    (11, 12),
    (12, 13),
]
vmo_flip_list = [
    VmoPart.Head,
    VmoPart.Neck,
    VmoPart.LShoulder,
    VmoPart.LElbow,
    VmoPart.LWrist,
    VmoPart.RShoulder,
    VmoPart.RElbow,
    VmoPart.RWrist,
    VmoPart.LHip,
    VmoPart.LKnee,
    VmoPart.LAnkle,
    VmoPart.RHip,
    VmoPart.RKnee,
    VmoPart.RAnkle,
]
# fmt: off
nr_vmo_links = 14
vmopose_vecs = list(
    zip(
        [1,1,8, 9, 1,11,12,1,2,3,1,5,6, 8],
        [0,8,9,10,11,12,13,2,3,4,5,6,7,11],
    )
)

nr_vmo16_links = 16
vmopose16_vecs = list(
    zip(
        [1,1,8, 9, 1,11,12,1,2,3,1,5,6, 8,2,5],
        [0,8,9,10,11,12,13,2,3,4,5,6,7,11,0,0],
    )
)

nr_vmo18_links = 18
vmopose18_vecs = list(
    zip(
        [1,1,8, 9, 1,11,12,1,2,3,1,5,6, 8,2,5,2, 5],
        [0,8,9,10,11,12,13,2,3,4,5,6,7,11,0,0,8,11],
    )
)
# fmt: on

nr_coco_keypoints = 17
coco_skeleton = [
    (15, 13),
    (13, 11),
    (16, 14),
    (14, 12),
    (11, 12),
    (5, 11),
    (6, 12),
    (5, 6),
    (5, 7),
    (6, 8),
    (7, 9),
    (8, 10),
    (1, 2),
    (0, 1),
    (0, 2),
    (1, 3),
    (2, 4),
    (3, 5),
    (4, 6),
]
cocoorg_flip_list = [
    CocoPart.Nose,
    CocoPart.REye,
    CocoPart.LEye,
    CocoPart.REar,
    CocoPart.LEar,
    CocoPart.RShoulder,
    CocoPart.LShoulder,
    CocoPart.RElbow,
    CocoPart.LElbow,
    CocoPart.RWrist,
    CocoPart.LWrist,
    CocoPart.RHip,
    CocoPart.LHip,
    CocoPart.RKnee,
    CocoPart.LKnee,
    CocoPart.RAnkle,
    CocoPart.LAnkle,
]

nr_openpose_keypoints = 18  # 18 joints
openpose_skeleton = [
    (1, 2),
    (1, 5),
    (2, 3),
    (3, 4),
    (5, 6),
    (6, 7),
    (1, 8),
    (8, 9),
    (9, 10),
    (1, 11),
    (11, 12),
    (12, 13),
    (1, 0),
    (0, 14),
    (14, 16),
    (0, 15),
    (15, 17),
]  # = 17
openpose_flip_list = [
    OpenposePart.Nose,
    OpenposePart.Neck,
    OpenposePart.LShoulder,
    OpenposePart.LElbow,
    OpenposePart.LWrist,
    OpenposePart.RShoulder,
    OpenposePart.RElbow,
    OpenposePart.RWrist,
    OpenposePart.LHip,
    OpenposePart.LKnee,
    OpenposePart.LAnkle,
    OpenposePart.RHip,
    OpenposePart.RKnee,
    OpenposePart.RAnkle,
    OpenposePart.LEye,
    OpenposePart.REye,
    OpenposePart.LEar,
    OpenposePart.REar,
]  # , OpenposePart.Background]

# fmt: off
nr_openpose_links = 19
openpose_vecs = list(
    zip(
        [1,8, 9, 1,11,12,1,2,3,1,5,6, 5,1, 0, 0,14, 2,15],
        [8,9,10,11,12,13,2,3,4,5,6,7,17,0,14,15,16,16,17],
    )
)

nr_openpose20_links = 20
openpose20_vecs = list(
    zip(
        [1,8, 9, 1,11,12, 8,1,2,3,1,5,6, 5,1, 0, 0,14, 2,15],
        [8,9,10,11,12,13,11,2,3,4,5,6,7,17,0,14,15,16,16,17],
    )
)

nr_openpose22_links = 22
openpose22_vecs = list(
    zip(
        [1,8, 9, 1,11,12, 8,2, 5,1,2,3,1,5,6, 5,1, 0, 0,14, 2,15],
        [8,9,10,11,12,13,11,8,11,2,3,4,5,6,7,17,0,14,15,16,16,17],
    )
)
# fmt: on



# IICO/VMO -- see IT, feel IT, love IT
# Copyright 2020 The IICO Corporate. All Rights Reserved.
#
# ==========================================================================
""" Image Data Utilities """

from typing import Union
from numpy import np


def color_image_shape_check(data: Union[list, np.ndarray]):
    """Check the shape of a list of image data

    Arguments:
        data {list/ndarray} -- Each of the element is
            in the shape of  `[height, width, depth]`.
            The `num_channel` should be the same for all elements
            in one of [3, 4]

    Raises:
        ValueError: If any of the input data is not in shape of
        `[height, width, depth]`.
        ValueError: If not all `depth` are the same.
    """
    if isinstance(data, np.ndarray):
        data = [data]
    for datum in data:
        if len(datum.shape) != 3:
            raise ValueError(
                "[color_image_shape_check]: Incorrect image data format!"
            )
        if data[0].shape[2] not in [3, 4]:
            raise ValueError(
                "[color_image_shape_check]: Incorrect image data format!"
            )
        if datum.shape[2] != data[0].shape[2]:
            raise ValueError(
                "[color_image_shape_check]: Inconsistent image depth!"
            )



# IICO/VMO -- see IT, feel IT, love IT
# Copyright 2020 The IICO Corporate. All Rights Reserved.
#
# ============================================================================
""" COCO Person Drawing Utilities """
from typing import Union

import cv2
import numpy as np
import PIL.Image as Image
import PIL.ImageDraw as ImageDraw
import PIL.ImageFont as ImageFont
from dataclasses import dataclass

from .keypoints import keypoint_array
from . import constants as vmoc
from .bbox import UniviewBBox

try:
    FONT = ImageFont.truetype("arial.ttf", 24)
except IOError:
    FONT = ImageFont.load_default()

palette = (2 ** 11 - 1, 2 ** 15 - 1, 2 ** 20 - 1)

CocoColors = [
    [255, 0, 0],
    [255, 85, 0],
    [255, 170, 0],
    [255, 255, 0],
    [170, 255, 0],
    [85, 255, 0],
    [0, 255, 0],
    [0, 255, 85],
    [0, 255, 170],
    [0, 255, 255],
    [0, 170, 255],
    [0, 85, 255],
    [0, 0, 255],
    [85, 0, 255],
    [170, 0, 255],
    [255, 0, 255],
    [255, 0, 170],
    [255, 0, 85],
    [255, 0, 0],
]


def compute_color_for_labels(label):
    """ palette by class label """
    color = [int((p * (label ** 2 - label + 1)) % 255) for p in palette]
    return tuple(color)


def _draw_single_box(
    image,
    xmin,
    ymin,
    xmax,
    ymax,
    color=(0, 255, 0),
    display_str=None,
    font=None,
    width=2,
    alpha=0.5,
    fill=False,
):
    if font is None:
        font = FONT

    draw = ImageDraw.Draw(image, mode="RGBA")
    left, right, top, bottom = xmin, xmax, ymin, ymax
    alpha_color = color + (int(255 * alpha),)
    draw.rectangle(
        [(left, top), (right, bottom)],
        outline=color,
        fill=alpha_color if fill else None,
        width=width,
    )

    if display_str:
        text_bottom = bottom
        # Reverse list and print from bottom to top.
        text_width, text_height = font.getsize(display_str)
        margin = np.ceil(0.05 * text_height)
        draw.rectangle(
            xy=[
                (left + width, text_bottom - text_height - 2 * margin - width),
                (left + text_width + width, text_bottom - width),
            ],
            fill=alpha_color,
        )
        draw.text(
            (
                left + margin + width,
                text_bottom - text_height - margin - width,
            ),
            display_str,
            fill="black",
            font=font,
        )

    return image


def draw_boxes(
    image: np.ndarray,
    boxes: Union[list, np.ndarray],
    labels: Union[list, np.ndarray] = None,
    scores: Union[list, np.ndarray] = None,
    class_name_map: Union[list, dict] = None,
    line_width: int = 2,
    alpha: float = 0.5,
    fill: bool = False,
    font=None,
    score_format=":{:.2f}",
):
    """Draw bboxes(labels, scores) on image
    Args:
        image: numpy array image, shape should be (height, width, channel)
        boxes: bboxes, shape should be (N, 4), and each row is
               (xmin, ymin, xmax, ymax) in ratios
        labels: labels, shape: (N, )
        scores: label scores, shape: (N, )
        class_name_map: list or dict, map class id to class name for
                        visualization.
        line_width: box line width
        alpha: text background alpha
        fill: fill box or not
        font: text font
        score_format: score format
    Returns:
        An image with information drawn on it.
    """
    imh, imw, _ = image.shape
    boxes = np.array(boxes)
    num_boxes = boxes.shape[0]
    if num_boxes > 0:
        boxes *= np.array([imw, imh, imw, imh])
    if isinstance(image, Image.Image):
        draw_image = image
    elif isinstance(image, np.ndarray):
        draw_image = Image.fromarray(image)
    else:
        raise AttributeError("Unsupported images type {}".format(type(image)))

    for i in range(num_boxes):
        display_str = ""
        color = (0, 255, 0)
        if labels is not None:
            this_class = labels[i]
            color = compute_color_for_labels(this_class)
            class_name = (
                class_name_map[this_class]
                if class_name_map is not None
                else str(this_class)
            )
            display_str = class_name

        if scores is not None:
            prob = scores[i]
            if display_str:
                display_str += score_format.format(prob)
            else:
                display_str += "score" + score_format.format(prob)

        draw_image = _draw_single_box(
            image=draw_image,
            xmin=boxes[i, 0],
            ymin=boxes[i, 1],
            xmax=boxes[i, 2],
            ymax=boxes[i, 3],
            color=color,
            display_str=display_str,
            font=font,
            width=line_width,
            alpha=alpha,
            fill=fill,
        )

    image = np.array(draw_image, dtype=np.uint8)
    return image


def find_contours(*args, **kwargs):
    """
    Wraps cv2.findContours to maintain compatibility between versions 3 and 4
    Returns:
        contours, hierarchy
    """
    if cv2.__version__.startswith("4"):
        contours, hierarchy = cv2.findContours(*args, **kwargs)
    elif cv2.__version__.startswith("3"):
        _, contours, hierarchy = cv2.findContours(*args, **kwargs)
    else:
        raise AssertionError(
            "cv2 must be either version 3 or 4 to call this method"
        )
    return contours, hierarchy


def draw_masks(
    image,
    masks,
    labels=None,
    border=True,
    border_width=2,
    border_color=(255, 255, 255),
    alpha=0.5,
    color=None,
):
    """
    Args:
        image: numpy array image, shape should be (height, width, channel)
        masks: (N, 1, Height, Width)
        labels: mask label
        border: draw border on mask
        border_width: border width
        border_color: border color
        alpha: mask alpha
        color: mask color
    Returns:
        np.ndarray
    """
    if isinstance(image, Image.Image):
        image = np.array(image)
    assert isinstance(image, np.ndarray)
    masks = np.array(masks)
    for i, mask in enumerate(masks):
        mask = mask.squeeze()[:, :, None].astype(np.bool)

        label = labels[i] if labels is not None else 1
        _color = (
            compute_color_for_labels(label) if color is None else tuple(color)
        )

        image = np.where(
            mask, mask * np.array(_color) * alpha + image * (1 - alpha), image
        )
        if border:
            contours, hierarchy = find_contours(
                mask.astype(np.uint8), cv2.RETR_TREE, cv2.CHAIN_APPROX_SIMPLE
            )
            image = cv2.drawContours(
                image,
                contours,
                -1,
                border_color,
                thickness=border_width,
                lineType=cv2.LINE_AA,
            )

    image = image.astype(np.uint8)
    return image


def vis_keypoints(
    image: np.ndarray,
    keypoints: Union[list, np.ndarray],
    diameter: int = 5,
    show_label: bool = True,
) -> np.ndarray:
    """ Draw person keypoints. Only valid and visible keypoints will be shown
    Args:
        image: numpy array image, shape should be (height, width, channel)
        keypoints: The keypoints input can be in many different formats:
                   1). list of long list, each of the sub-list is a long
                       repetitive [x, y, v, ...]
                   2). list of list of keypoints, each of the sub-list is a
                       list of (x, y, v) keypoints.
                   3). ndarray of shape [n_instance, n_keypoints, 3]
                   4). list of ndarray of shape [n_keypoints, 3]
                   5). list of list of tuples in (x, y)
        diameter: radius of keypoint circle
        show_label: mark keypoint label next to joint
    Returns:
        np.ndarray
    """
    image = image.copy()
    keypoints = keypoint_array(keypoints)
    if keypoints.shape[0] > 0 and len(keypoints.shape) == 3:
        n_inst, n_kps, _ = keypoints.shape
        for i in range(n_inst):
            for j in range(n_kps):
                if keypoints.shape[2] == 3:
                    x, y, v = keypoints[i, j, :]
                    if v < 2:
                        continue
                if keypoints.shape[2] == 2:
                    x, y = keypoints[i, j, :]
                    if x < 0 or y < 0:
                        continue
                cv2.circle(
                    image, (int(x), int(y)), diameter, CocoColors[j], -1
                )
                if show_label:
                    cv2.putText(
                        image,
                        str(j),
                        (int(x) + 5, int(y)),
                        cv2.FONT_HERSHEY_COMPLEX_SMALL,
                        0.7,
                        (255, 255, 255),
                    )
    return image


def viz_vmo_annotation(
    image: np.ndarray,
    bboxes: Union[list, np.ndarray] = None,
    keypoints: Union[list, np.ndarray] = None,
    labels: Union[list, np.ndarray] = None,
    box_format: str = "albu",
    resize: float = 1.0,
) -> np.ndarray:
    """Visualize vmo example data with person annotations

    Args:
        image: np.ndarray, image data in RGB format
        box_format: one of ['coco', 'yolo', 'albu']
        bboxes: list or np.ndarray
        categories: np.ndarray,
        keypoints: np.ndarray,
        r_resize: if not equals to 1.0, resize image to target ratio

    Retruns:
        display image
    """
    im = np.copy(image[:, :, ::-1])
    if bboxes is not None:
        if not box_format == "albu":
            bboxes = UniviewBBox().get_albu_bbox(bboxes, box_format, im.shape)
        im = draw_boxes(im, bboxes, labels)
        im = vis_keypoints(im, keypoints)
    if resize != 1.0:
        im = cv2.resize(im, (0, 0), fx=resize, fy=resize)
    return im


def viz_vmo_pose(
    vmo_example: dataclass, show_box: bool = True, r_resize: float = 1.0
) -> np.ndarray:
    """Visualize vmo example data with person skeletions

    Args:
        vmo_example: a dataclass object that has the following
        necessary fields:
        - imdata: np.ndarray, image data in RGB format
        - bboxes: np.ndarray, bounding boxes in albumentation format
        - categories: np.ndarray,
        - keypoints: np.ndarray,
        - keypoint_format: str, keypoint format in ['vmo', 'torso']
        show_box: if True, display bounding box and category label
        r_resize: if not equals to 1.0, resize image to target ratio

    Retruns:
        display image
    """
    if vmo_example.keypoint_format not in ["vmo", "torso"]:
        raise Exception("//Error: please provide valid vmo keypoint format!")
    if vmo_example.keypoint_format == "vmo":
        _skeleton = vmoc.vmo_skeleton
    elif vmo_example.keypoint_format == "torso":
        _skeleton = vmoc.torso_skeleton
    else:
        raise Exception("//Error: undefined skeleton format!")

    imdsp = np.copy(vmo_example.imdata[:, :, ::-1])
    if show_box:
        imdsp = draw_boxes(imdsp, vmo_example.bboxes, vmo_example.categories)
    imdsp = vis_keypoints(imdsp, vmo_example.keypoints)

    kps_foramt = None
    if vmo_example.keypoints.shape[0] > 0:
        if vmo_example.keypoints.shape[2] == 3:
            kps_foramt = "xyv"
        elif vmo_example.keypoints.shape[2] == 2:
            kps_foramt = "xy"
        else:
            raise Exception("//Error: unknown keypoint format!")
    for _inst in vmo_example.keypoints:
        for p, q in _skeleton:
            if kps_foramt == "xyv":
                x0, y0, v0 = _inst[p]
                x1, y1, v1 = _inst[q]
                if v0 < 2 or v1 < 2:
                    continue
            if kps_foramt == "xy":
                x0, y0 = _inst[p]
                x1, y1 = _inst[q]
                if x0 < 0 or y0 < 0 or x1 < 0 or y1 < 0:
                    continue
            cv2.line(
                imdsp, (int(x0), int(y0)), (int(x1), int(y1)), CocoColors[p], 2
            )

    if r_resize != 1.0:
        imdsp = cv2.resize(imdsp, (0, 0), fx=r_resize, fy=r_resize)

    return imdsp


def draw_humans(
    img: np.ndarray, human_list: list, joint_format: str = "torso"
) -> np.ndarray:
    """Draw human pose skeleton
    Using torse skeleton as example, each human in the data structure
    of human_list is like:
    defaultdict {
        0: (0, (0.14375, 0.42990654205607476), 1.0),
        1: (1, (0.10625, 0.48598130841121495), 1.0),
        2: (2, (0.13125, 0.48598130841121495), 1.0),
        3: (3, (0.1, 0.616822429906542), 1.0)
    }
    """
    if "vmo" in joint_format:
        nr_keypoints = vmoc.nr_vmo_keypoints
        if joint_format == "vmo":
            _Pairs = vmoc.vmopose_vecs
        elif joint_format == "vmo16":
            _Pairs = vmoc.vmopose16_vecs
    elif joint_format == "torso":
        nr_keypoints = vmoc.nr_torso_keypoints
        _Pairs = vmoc.torsopose_vecs
    else:
        print("-- Undefined pose data format!")

    img_copied = np.copy(img)
    image_h, image_w = img_copied.shape[:2]
    centers = {}

    radius = 2
    cthickness = 3
    lthickness = 2

    for human in human_list:
        part_idxs = human.keys()

        # draw point
        for i in range(nr_keypoints):
            if i not in part_idxs:
                continue
            part_coord = human[i][1]
            center = (
                int(part_coord[0] * image_w + 0.5),
                int(part_coord[1] * image_h + 0.5),
            )
            centers[i] = center
            cv2.circle(
                img_copied,
                center,
                radius,
                CocoColors[i],
                thickness=cthickness,
                lineType=8,
                shift=0,
            )

        # draw line
        for pair_order, pair in enumerate(_Pairs):
            if pair[0] not in part_idxs or pair[1] not in part_idxs:
                continue
            img_copied = cv2.line(
                img_copied,
                centers[pair[0]],
                centers[pair[1]],
                CocoColors[pair_order],
                lthickness,
            )

    return img_copied



# IICO/VMO -- see IT, feel IT, love IT
# Copyright 2020 The IICO Corporate. All Rights Reserved.
#
# ============================================================================
""" 2D Person Keypoint Utilities """

from typing import Union
import numpy as np


def keypoint_array(keypoints: Union[list, np.ndarray]) -> np.ndarray:
    """Convert keypoints to ndarray

    Args:
        keypoints: The keypoints input can be in many different formats:
                   1). list of long list, each of the sub-list is a long
                       repetitive [x, y, v, ...]
                   2). list of list of keypoints, each of the sub-list is a
                       list of (x, y, v) keypoints.
                   3). ndarray of shape [n_instance, n_keypoints, 3]
                   4). list of ndarray of shape [n_keypoints, 3]
                   5). list of list of tuples in (x, y)

    Returns:
        np.ndarray in the format of [n_instance, n_keypoints, 3], where
        the 3 keypoint data are (x, y, v) with v for quality value.
    """
    keypoints = np.array(keypoints)
    if keypoints.shape[0] > 0:
        if len(keypoints.shape) not in [2, 3]:
            raise Exception("// Error: keypoint data in wrong data shape!")
        if len(keypoints.shape) == 2 and keypoints.shape[1] % 3 == 0:
            keypoints = keypoints.reshape((keypoints.shape[0], -1, 3))
    return keypoints


def coco_2_openpose(coco_kps: Union[list, np.ndarray]) -> np.ndarray:
    """ convert from coco keypoint format to openpose format

    HY: based on the 17 COCO annotated keypoints, here the keypoints are
    remapped to revised openpose sequence.

    Furthermore, there adds a new keypoint, which marks the middle point
    between the 2 shoulder keypoints. This new keypoint is labelled as
    the 2nd keypoint in the new sequence.


    Args:
        coco_kps: array of keypoints in coco format

    Returns:
        array of keypoints in revised openpose-18 format
    """
    coco_kps = keypoint_array(coco_kps)
    pt18_joint_list = []
    transform = list(
        zip(
            [0, 5, 6, 8, 10, 5, 7, 9, 12, 14, 16, 11, 13, 15, 2, 1, 4, 3],
            [0, 6, 6, 8, 10, 5, 7, 9, 12, 14, 16, 11, 13, 15, 2, 1, 4, 3],
        )
    )
    for prev_joint in coco_kps:
        new_joint = []
        for idx1, idx2 in transform:
            j1 = prev_joint[idx1]
            j2 = prev_joint[idx2]
            new_joint.append(
                ((j1[0] + j2[0]) / 2, (j1[1] + j2[1]) / 2, min(j1[2], j2[2]))
            )
        pt18_joint_list.append(new_joint)
    return np.array(pt18_joint_list)


def coco_2_vmo(coco_kps: Union[list, np.ndarray]) -> np.ndarray:
    """ convert from coco keypoint format to vmo-14 format

    VMO has 14 keypoints. Here based on the 17 COCO annotated keypoints,
    the keypoints are remapped with new sequence.
    1). Add one NECK keypoint, which marks the middle point between
        the 2 should keypoints.
    2). Find Head as the center of available (Nose), Eyes, and Ears.

    Args:
        coco_kps: array of keypoints in coco format

    Returns:
        array of keypoints in vmo-14 format
    """
    coco_kps = keypoint_array(coco_kps)
    pt14_joint_list = []
    transform = list(
        zip(
            [0, 5, 6, 8, 10, 5, 7, 9, 12, 14, 16, 11, 13, 15],
            [0, 6, 6, 8, 10, 5, 7, 9, 12, 14, 16, 11, 13, 15],
        )
    )
    for prev_joint in coco_kps:
        new_joint = []
        for idx1, idx2 in transform:
            j1 = prev_joint[idx1]
            j2 = prev_joint[idx2]
            new_joint.append(
                ((j1[0] + j2[0]) / 2, (j1[1] + j2[1]) / 2, min(j1[2], j2[2]))
            )

        # -: reprocess head as the center of nose, eyes, and ears
        head_parts_x = []
        head_parts_y = []
        for i in range(1, 5):  # 0:nose (removed); 1,2: eyes; 3,4: ears in coco
            jt = prev_joint[i]
            if jt[2] >= 2:
                head_parts_x.append(jt[0])
                head_parts_y.append(jt[1])

        if head_parts_x and head_parts_y:
            new_joint[0] = (
                int(sum(head_parts_x) / float(len(head_parts_x))),
                int(sum(head_parts_y) / float(len(head_parts_y))),
                2,
            )
        else:
            new_joint[0] = (-1000, -1000, 0)

        # -: add skeleton instance
        pt14_joint_list.append(new_joint)

    return np.array(pt14_joint_list)


def coco13_2_vmo(coco_kps: Union[list, np.ndarray]) -> np.ndarray:
    """ convert from coco-13 keypoint format to vmo-14 format

    VMO has 14 keypoints. Here based on the revised COCO-13 keypoints,
    the keypoints are remapped with new sequence.
    1). Add one NECK keypoint, which marks the middle point between
        the 2 should keypoints.

    Args:
        coco_kps: array of keypoints in coco-13 format

    Returns:
        array of keypoints in vmo-14 format
    """
    coco_kps = keypoint_array(coco_kps)
    pt14_joint_list = []
    transform = list(
        zip(
            [0, 1, 2, 4, 6, 1, 3, 5, 8, 10, 12, 7, 9, 11],
            [0, 2, 2, 4, 6, 1, 3, 5, 8, 10, 12, 7, 9, 11],
        )
    )
    for prev_joint in coco_kps:
        new_joint = []
        for idx1, idx2 in transform:
            j1 = prev_joint[idx1]
            j2 = prev_joint[idx2]
            new_joint.append(
                ((j1[0] + j2[0]) / 2, (j1[1] + j2[1]) / 2, min(j1[2], j2[2]))
            )
        pt14_joint_list.append(new_joint)

    return np.array(pt14_joint_list)


def coco13_2_torso(coco_kps: Union[list, np.ndarray]) -> np.ndarray:
    """ convert from coco-13 keypoint format to torso-4 format

    Torso-4 has 4 keypoints. Here based on the revised COCO-13 keypoints,
    the keypoints are remapped with new sequence.

    Args:
        coco_kps: array of keypoints in coco-13 format

    Returns:
        array of keypoints in torse-4 format
    """
    coco_kps = keypoint_array(coco_kps)
    pt4_joint_list = []
    transform = list(zip([0, 2, 1, 7], [0, 2, 1, 8]))
    for prev_joint in coco_kps:
        new_joint = []
        for idx1, idx2 in transform:
            j1 = prev_joint[idx1]
            j2 = prev_joint[idx2]
            new_joint.append(
                ((j1[0] + j2[0]) / 2, (j1[1] + j2[1]) / 2, min(j1[2], j2[2]))
            )
        pt4_joint_list.append(new_joint)

    return np.array(pt4_joint_list)


SUPPORTED_FORMATS = ["coco", "coco13", "vmo", "torso"]


CONVERSIONS_Registry = {
    "coco-openpose": coco_2_openpose,
    "coco-vmo": coco_2_vmo,
    "coco13-vmo": coco13_2_vmo,
    "coco13-torso": coco13_2_torso,
}



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['uniview', 'uniview.im2d', 'uniview.vmo']

package_data = \
{'': ['*']}

install_requires = \
['Pillow>=7.2.0,<8.0.0',
 'absl-py>=0.10.0,<0.11.0',
 'numpy>=1.19.2,<2.0.0',
 'opencv-python==4.2.0.34']

entry_points = \
{'console_scripts': ['run = uniview:main']}

setup_kwargs = {
    'name': 'uniview',
    'version': '0.1.1',
    'description': 'Datapoint Visualization and Illustration Toolkit',
    'long_description': '',
    'author': 'H-AI',
    'author_email': 'dongjian413@gmail.com',
    'maintainer': 'H-AI',
    'maintainer_email': 'dongjian413@gmail.com',
    'url': '',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
